TYPE
  LOGLEVEL : (CRITICAL, WARNING, INFO, DEBUG) := INFO;
END_TYPE

FUNCTION_BLOCK LOGGER
  VAR_INPUT
    TRIG : BOOL;
    MSG : STRING;
    LEVEL : LOGLEVEL := INFO;
  END_VAR
  VAR
    TRIG0 : BOOL;
  END_VAR

  IF TRIG AND NOT TRIG0 THEN
  {{
   LogMessage(GetFbVar(LEVEL),(char*)GetFbVar(MSG, .body),GetFbVar(MSG, .len));
  }}
  END_IF;
  TRIG0:=TRIG;
END_FUNCTION_BLOCK



FUNCTION_BLOCK python_eval
  VAR_INPUT
    TRIG : BOOL;
    CODE : STRING;
  END_VAR
  VAR_OUTPUT
    ACK : BOOL;
    RESULT : STRING;
  END_VAR
  VAR
    STATE : DWORD;
    BUFFER : STRING;
    PREBUFFER : STRING;
    TRIGM1 : BOOL;
    TRIGGED : BOOL;
  END_VAR

  {extern void __PythonEvalFB(int, PYTHON_EVAL*);__PythonEvalFB(0, data__);}
END_FUNCTION_BLOCK

FUNCTION_BLOCK python_poll
  VAR_INPUT
    TRIG : BOOL;
    CODE : STRING;
  END_VAR
  VAR_OUTPUT
    ACK : BOOL;
    RESULT : STRING;
  END_VAR
  VAR
    STATE : DWORD;
    BUFFER : STRING;
    PREBUFFER : STRING;
    TRIGM1 : BOOL;
    TRIGGED : BOOL;
  END_VAR

  {extern void __PythonEvalFB(int, PYTHON_EVAL*);__PythonEvalFB(1,(PYTHON_EVAL*)(void*)data__);}
END_FUNCTION_BLOCK

FUNCTION_BLOCK python_gear
  VAR_INPUT
    N : UINT;
    TRIG : BOOL;
    CODE : STRING;
  END_VAR
  VAR_OUTPUT
    ACK : BOOL;
    RESULT : STRING;
  END_VAR
  VAR
    py_eval : python_eval;
    COUNTER : UINT;
    ADD10_OUT : UINT;
    EQ13_OUT : BOOL;
    SEL15_OUT : UINT;
    AND7_OUT : BOOL;
  END_VAR

  ADD10_OUT := ADD(COUNTER, 1);
  EQ13_OUT := EQ(N, ADD10_OUT);
  SEL15_OUT := SEL(EQ13_OUT, ADD10_OUT, 0);
  COUNTER := SEL15_OUT;
  AND7_OUT := AND(EQ13_OUT, TRIG);
  py_eval(TRIG := AND7_OUT, CODE := CODE);
  ACK := py_eval.ACK;
  RESULT := py_eval.RESULT;
END_FUNCTION_BLOCK


FUNCTION_BLOCK GeneratoreOndaQuadra
  VAR_INPUT
    Input : BOOL;
  END_VAR
  VAR_OUTPUT
    Output : BOOL;
  END_VAR
  VAR
    R_TRIG1 : R_TRIG;
    R_TRIG2 : R_TRIG;
    R_TRIG3 : R_TRIG;
  END_VAR

  INITIAL_STEP Quiete:
    COMPUTE_FUNCTION_BLOCKS(S);
  END_STEP

  ACTION COMPUTE_FUNCTION_BLOCKS:
    R_TRIG1(CLK := Input);
    R_TRIG2(CLK := Input);
    R_TRIG3(CLK := Input);
  END_ACTION

  TRANSITION FROM Quiete TO Onda_On
    := R_TRIG1.Q;
  END_TRANSITION

  STEP Onda_On:
    Output(N);
  END_STEP

  TRANSITION FROM Onda_On TO Quiete
    := R_TRIG2.Q;
  END_TRANSITION

  TRANSITION FROM Onda_On TO Onda_Off
    := (Onda_On.T>=T#4s) AND (NOT(EDGE_R(CLK:=Input).Q));
  END_TRANSITION

  STEP Onda_Off:
  END_STEP

  TRANSITION FROM Onda_Off TO Quiete
    := R_TRIG3.Q;
  END_TRANSITION

  TRANSITION FROM Onda_Off TO Onda_On
    := (Onda_Off.T>=T#6s) AND (NOT(Input));
  END_TRANSITION

END_FUNCTION_BLOCK

PROGRAM Main
  VAR
    IN : BOOL;
    OUT : BOOL;
    OndaQuadra : GeneratoreOndaQuadra;
  END_VAR

  OndaQuadra(Input:=IN);
  OUT:=OndaQuadra.Output;
END_PROGRAM


CONFIGURATION Config0

  RESOURCE Res0 ON PLC
    TASK CicloDiScansione(INTERVAL := T#100ms,PRIORITY := 0);
    PROGRAM InstanzaMain WITH CicloDiScansione : Main;
  END_RESOURCE
END_CONFIGURATION
